package com.xs.core.model;

import lombok.Data;
import org.springframework.context.MessageSource;
import org.springframework.context.i18n.LocaleContextHolder;

import java.io.Serializable;
import java.util.Locale;

@Data
public class ResponseResult<T> implements Serializable {
    public static final long serialVersionUID = 42L;

    private Integer code;
    private String msg;
    private T data;

    private static MessageSource messageSource;

    public static void setMessageSource(MessageSource source) {
        messageSource = source;
    }

    public static String getMessage(String key, Object... args) {
        Locale locale = LocaleContextHolder.getLocale();
        return messageSource.getMessage(key, args, locale);
    }

    protected ResponseResult() {
    }

    protected ResponseResult(IErrorCode errorCode, String msg) {
        this.code = errorCode.getCode();
        this.msg = msg;
    }

    protected ResponseResult(Integer code, String msgKey, Object... args) {
        this.code = code;
        this.msg = getMessage(msgKey, args);
    }

    protected ResponseResult(Integer code, String msgKey, T data, Object... args) {
        this.code = code;
        this.msg = getMessage(msgKey, args);
        this.data = data;
    }

    protected ResponseResult(Integer code, String msgKey, T data) {
        this.code = code;
        this.msg = getMessage(msgKey);
        this.data = data;
    }

    public static <T> ResponseResult<T> success() {
        return new ResponseResult<>(ResultCode.SUCCESS.getCode(), "response.success", null);
    }

    public static <T> ResponseResult<T> success(T data) {
        return new ResponseResult<>(ResultCode.SUCCESS.getCode(), "response.success", data);
    }

    public static <T> ResponseResult<T> success(T data, String msgKey) {
        return new ResponseResult<>(ResultCode.SUCCESS.getCode(), msgKey, data);
    }

    public static <T> ResponseResult<T> success(T data, String msgKey, Object... args) {
        return new ResponseResult<>(ResultCode.SUCCESS.getCode(), msgKey, data, args);
    }

    public static <T> ResponseResult<T> success(String msgKey, Object... args) {
        return new ResponseResult<>(ResultCode.SUCCESS.getCode(), msgKey, args);
    }

    public static <T> ResponseResult<T> failed(IErrorCode errorCode, String msgKey, Object... args) {
        return new ResponseResult<>(errorCode.getCode(), msgKey, args);
    }

    public static <T> ResponseResult<T> failed(String msgKey, Object... args) {
        return new ResponseResult<>(ResultCode.FAILED.getCode(), msgKey, args);
    }

    public static <T> ResponseResult<T> failed(String msg) {
        return new ResponseResult<>(ResultCode.FAILED, msg);
    }

    public static <T> ResponseResult<T> failed() {
        return failed(ResultCode.FAILED, "response.failed");
    }

    public static <T> ResponseResult<T> unauthorized(T data) {
        return new ResponseResult<>(ResultCode.UNAUTHORIZED.getCode(), "response.unauthorized", data);
    }

    public static <T> ResponseResult<T> forbidden(T data) {
        return new ResponseResult<>(ResultCode.FORBIDDEN.getCode(), "response.forbidden", data);
    }

    public static <T> ResponseResult<T> forbidden() {
        return forbidden(null);
    }

    public static <T> ResponseResult<T> badRequest(T data) {
        return new ResponseResult<>(ResultCode.BAD_REQUEST.getCode(), "response.bad.request", data);
    }

    public static <T> ResponseResult<T> badRequest() {
        return badRequest(null);
    }

    public static <T> ResponseResult<T> requestTooLarge(T data) {
        return new ResponseResult<>(ResultCode.REQUEST_ENTITY_TOO_LARGE.getCode(), "response.request.too.large", data);
    }

    public static <T> ResponseResult<T> unsupportedMediaType(T data) {
        return new ResponseResult<>(ResultCode.UNSUPPORTED_MEDIA_TYPE.getCode(), "response.unsupported.media.type", data);
    }

    public static <T> ResponseResult<T> serverError(T data) {
        return new ResponseResult<>(ResultCode.SERVER_ERROR.getCode(), "response.server.error", data);
    }

    public static <T> ResponseResult<T> notFound() {
        return new ResponseResult<>(ResultCode.NOT_FOUND.getCode(), "response.not.found", null);
    }
}